<?php

namespace Drupal\pagedesigner_embed\Plugin\pagedesigner\Handler;

use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner_media\Plugin\MediaFieldHandlerBase;

/**
 * Process entities of type "embed".
 *
 * @PagedesignerHandler(
 *   id = "embed",
 *   name = @Translation("Embed handler"),
 *   types = {
 *      "embed"
 *   },
 * )
 */
class Embed extends MediaFieldHandlerBase {

  /**
   * {@inheritdoc}
   */
  public function collectAttachments(array &$attachments) {
    $attachments['library'][] = 'pagedesigner_embed/pagedesigner';
  }

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $data = [
      'src' => '',
      'id' => $entity->field_media->target_id,
    ];
    $data['src'] = $this->elementHandler->get($entity);
    $result = $data + $result;

  }

  /**
   * {@inheritdoc}
   */
  public function get(Element $entity, string &$result = '') {
    if ($entity->field_media->entity != NULL && $entity->field_media->entity->field_embed_source != NULL) {
      $provider = \Drupal::service('video_embed_field.provider_manager')->loadProviderFromInput($entity->field_media->entity->field_embed_source->value);
      $renderArray = $provider->renderEmbedCode(0, 0, 0);
      $result = $renderArray['#url'];
    }
  }

  /**
   * {@inheritdoc}
   */
  public function generate($definition, array $data, Element &$entity = NULL) {
    parent::generate(['type' => 'embed', 'name' => 'embed'], $data, $entity);
  }

}
