<?php

namespace Drupal\pagedesigner_link\Plugin\pagedesigner\Handler;

use Drupal\Core\Url;
use Drupal\pagedesigner\Entity\Element;
use Drupal\pagedesigner\Plugin\PlainFieldHandlerBase;
use Drupal\Core\Language\LanguageInterface;

/**
 * Process entities of type "link".
 *
 * @PagedesignerHandler(
 *   id = "link",
 *   name = @Translation("Link handler"),
 *   types = {
 *      "link",
 *      "href",
 *   },
 * )
 */
class Link extends PlainFieldHandlerBase {

  /**
   * {@inheritdoc}
   */
  public function collectAttachments(array &$attachments) {
    $attachments['library'][] = 'pagedesigner_link/pagedesigner';
  }

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $result = [
      'uri' => $entity->field_link->uri,
      'title' => $entity->field_link->title,
    ] + $result;
  }

  /**
   * {@inheritdoc}
   */
  public function get(Element $entity, string &$result = '') {
    $result = $entity->field_link->uri;
    if (strpos($result, '/') === 0) {
      $url = Url::fromUri("internal:" . $result);
      if ($url->isRouted()) {
        $params = $url->getRouteParameters();
        $entity_type = key($params);
        if ($entity_type == 'media') {
          $mediaEntity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($params[$entity_type]);
          if ($mediaEntity != NULL) {
            $url = \Drupal::service('plugin.manager.linkit.substitution')
              ->createInstance('media')
              ->getUrl($mediaEntity);
            if ($url) {
              $result = $url->getGeneratedUrl();
            }
          }
        }
        elseif ($entity_type == 'node' && preg_match('/\/node\/[0-9]+/', $result)) {
          $langcode = $this->languageManager
            ->getCurrentLanguage(LanguageInterface::TYPE_CONTENT)
            ->getId();
          $result = '/' . $langcode . \Drupal::service('path_alias.manager')->getAliasByPath($result, $langcode);
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getContent(Element $entity, array &$list = []) {

  }

  /**
   * {@inheritdoc}
   */
  public function render(Element $entity, array &$build = []) {
    $build = [];
    parent::render($entity, $build);
    if (strpos($entity->field_link->uri, '/') === 0) {
      $url = Url::fromUri("internal:" . $entity->field_link->uri);
      if ($url->isRouted()) {
        $params = $url->getRouteParameters();
        $entity_type = key($params);
        if ($entity_type == 'media') {
          $mediaEntity = \Drupal::entityTypeManager()->getStorage($entity_type)->load($params[$entity_type]);
          if ($mediaEntity != NULL) {
            \Drupal::service('renderer')->addCacheableDependency($build, $mediaEntity);
          }
        }
      }
    }
  }

  /**
   * {@inheritdoc}
   */
  public function patch(Element $entity, array $data) {
    if (!empty($data['uri']) || $data['id'] == NULL) {
      $entity->field_link->uri = $data['uri'];
      $entity->field_link->title = $data['title'];
      $entity->saveEdit();
    }
  }

  /**
   * {@inheritdoc}
   */
  public function generate($definition, array $data, Element &$entity = NULL) {
    parent::generate(['type' => 'link', 'name' => 'link'], $data, $entity);
  }

}
