<?php

namespace Drupal\pagedesigner\EventSubscriber;

use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\GetResponseEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Routing\CurrentRouteMatch;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Url;

/**
 * Event subscriber to disable css and js aggregation in editing mode.
 *
 * In editing mode, the pagedesigner needs to
 * selectively load javascript and css. Aggregation would prevent this.
 */
class InitSubscriber implements EventSubscriberInterface {

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountProxyInterface
   */
  protected $user = NULL;


  /**
   * The current route.
   *
   * @var \Drupal\Core\Routing\CurrentRouteMatch
   */
  protected $_currentRoute = NULL;

  /**
   * Create the InitSubscriber.
   *
   * @param \Drupal\Core\Session\AccountProxyInterface $user
   *   The current user.
   * @param \Drupal\Core\Routing\CurrentRouteMatch $route
   *   The current route.
   */
  public function __construct(AccountProxyInterface $user, CurrentRouteMatch $currentRoute) {
    $this->user = $user;
    $this->_currentRoute = $currentRoute;
  }

  /**
   * Redirect old pagedesigner url to new route.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   The event.
   */
  public function redirectOldUrl(GetResponseEvent $event) {

    if (!$this->user->isAnonymous() && \Drupal::request()->query->get('pd') == 1) {
      $nid = $this->_currentRoute->getRawParameter('node');
      if (\is_numeric($nid)) {
        $event->setResponse(new RedirectResponse(Url::fromRoute('pagedesigner.node.edit_mode', ['node' => $nid])->toString()));
      }
    }
  }

  /**
   * Disable aggregation in pagedesigner editing mode.
   *
   * @param \Symfony\Component\HttpKernel\Event\GetResponseEvent $event
   *   The event.
   */
  public function disableAggregation(GetResponseEvent $event) {
    global $config;
    $editMode = \Drupal::request()->query->get('pd');
    if (!$this->user->isAnonymous() && $editMode == 1) {
      // $config['system.performance']['css']['preprocess'] = FALSE;
      // $config['system.performance']['js']['preprocess'] = FALSE;
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::REQUEST => [['redirectOldUrl', 30], ['disableAggregation', 255]],
    ];
  }

}
