<?php

namespace Drupal\pagedesigner\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;

/**
 * Plugin implementation of the Pagedesigner content formatter.
 *
 * @FieldFormatter(
 *   id = "pagedesigner_formatter",
 *   module = "pagedesigner",
 *   label = @Translation("Pagedesigner content"),
 *   field_types = {
 *     "pagedesigner_item"
 *   }
 * )
 */
class PagedesignerFormatter extends FormatterBase {

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    $summary = [];
    $summary[] = $this->t('Displays the pagedesigner content.');
    return $summary;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $elements = [];
    $node = $items->getEntity();
    $pagedesignerService = \Drupal::service('pagedesigner.service');
    $renderer = \Drupal::service('pagedesigner.service.renderer');
    if ($node != NULL) {
      $node = $node->getTranslation($langcode);
      foreach ($items as $delta => $item) {
        $container = $item->entity;
        if ($container != NULL && $container->hasTranslation($langcode)) {
          $container = $container->getTranslation($langcode);
          if ($pagedesignerService->isPagedesignerRoute()) {
            $renderer->renderForEdit($container, $node);
          }
          elseif (\Drupal::currentUser()->hasPermission('view unpublished pagedesigner element entities')) {
            $renderer->render($container, $node);
          }
          else {
            $renderer->renderForPublic($container, $node);
          }
          $elements[] = $renderer->getOutput();
        }
      }
    }
    return $elements;
  }

}
