<?php

namespace Drupal\pagedesigner\Plugin\pagedesigner\Handler;

use Drupal\editor\Entity\Editor;
use Drupal\pagedesigner\Entity\Element;

/**
 * Process entities of type "html".
 *
 * @PagedesignerHandler(
 *   id = "html",
 *   name = @Translation("Html renderer"),
 *   types = {
 *     "html"
 *   },
 * )
 */
class Html extends Content {

  /**
   * {@inheritdoc}
   */
  public function serialize(Element $entity, array &$result = []) {
    $result = [
      $entity->field_content->value,
    ] + $result;
  }

  /**
   * {@inheritdoc}
   */
  public function get(Element $entity, string &$result = '') {
    $build = [];
    $this->render($entity, $build);
    $result .= \Drupal::service('renderer')
      ->renderPlain($build);
  }

  /**
   * {@inheritdoc}
   */
  public function render(Element $entity, array &$build = []) {
    $format = \Drupal::config('pagedesigner.settings')->get('filter_format');
    if (empty($format)) {
      $format = 'plain_text';
    }
    $build = [
      '#type' => 'processed_text',
      '#text' => $entity->field_content->value,
      '#format' => $format,
      '#filter_types_to_skip' => [],
      '#langcode' => $entity->langcode->value,
    ];
    \Drupal::service('renderer')->addCacheableDependency($build, $entity);
  }

}
