<?php

namespace Drupal\Tests\pagedesigner\Kernel\HandlerTests;

/**
 * Base class to test a compound handler with a predefined entity defintion.
 *
 * Tests serializing, getting and rendering of the entity.
 *
 * @group pagedesigner
 */
abstract class CompoundHandlerTestBase extends HandlerTestBase {

  /**
   * Test the render function of the handler.
   *
   * @return array
   *   The produced render representation of the entity.
   */
  protected function renderTest() {
    $build = parent::renderTest();
    $this->assertTrue(!empty($build['#lazy_builder']) || !empty($build['#pagedesigner_entity']));
    if (!empty($build['#pagedesigner_entity'])) {
      $this->assertTrue($build['#pagedesigner_entity']->id() == $this->entity->id());
    }
    return $build;
  }

  /**
   * Test the render function of the handler.
   *
   * @return array
   *   The produced render representation of the entity.
   */
  protected function renderForEditTest() {
    $build = parent::renderForEditTest();
    if (!empty($build['#type']) && $build['#type'] == 'pattern') {
      $this->assertTrue(!empty($build['#pagedesigner_entity']));
      $this->assertTrue(!empty($build['#pagedesigner_edit']));
      $this->assertTrue($build['#pagedesigner_entity']->id() == $this->entity->id());
      $this->assertTrue($build['#pagedesigner_edit']);
    }
    elseif (!empty($build['#type']) && $build['#type'] == 'inline_template') {
      $this->assertTrue(!empty($build['#template']));
      $this->assertTrue(strpos($build['#template'], 'data-entity-id="{{id}}"') !== FALSE);
      $this->assertTrue(strpos($build['#template'], 'id="{{html_id}}"') !== FALSE);
      $this->assertTrue(strpos($build['#template'], 'data-gjs-type="' . $this->handlerId) !== FALSE);
      $this->assertTrue($build['#context']['id'] === $this->entity->id());
      $this->assertTrue($build['#context']['html_id'] === 'pd-cp-' . $this->entity->id());
    }
    elseif (!empty($build['#prefix'])) {
      $this->assertTrue(strpos($build['#prefix'], 'data-entity-id="' . $this->entity->id() . '"') !== FALSE);
      $this->assertTrue(strpos($build['#prefix'], 'id="pd-cp-' . $this->entity->id() . '"') !== FALSE);
      $this->assertTrue(strpos($build['#prefix'], 'data-gjs-type="' . $this->handlerId) !== FALSE);
    }

    return $build;
  }

}
